from stat import S_IFDIR, S_IFLNK, S_IFREG, S_IFIFO
from time import time
import logging
import errno
from fuse import FuseOSError
from path import KubePath

class KubeFileSystem(object):
    def __init__(self, client):
        self.client = client
        self.open_files = {}
        self.flushed = {}

    def _stat_dir(self):
        return dict(st_mode=(S_IFDIR | 0o555), st_nlink=2,
                st_size=0, st_ctime=time(), st_mtime=time(),
                st_atime=time())

    def _stat_file(self, client, path, size=None):
        if size is None:
            size = len(path.do_action(client))
        ts = path.get_creation_date_for_action_file(client)
        if ts is None:
            ts = time()
        return dict(st_mode=(S_IFREG | path.get_mode()), st_nlink=1,
                st_size=size, st_ctime=ts, st_mtime=ts,
                st_atime=ts)

    def open_for_writing(self, path):
        if path not in self.open_files:
            self.open_files[path] = KubePath().parse_path(path).do_action(self.client)

    def open(self, path, fh):
        pass

    def truncate(self, path, length):
        self.open_for_writing(path)
        self.open_files[path] = self.open_files[path][:length]

    def write(self, path, buf, offset):
        self.open_for_writing(path)
        self.open_files[path] = self.open_files[path][:offset] + buf
        return len(buf)

    def close(self, path):
        if path not in self.open_files:
            return
        self.persist(path, self.open_files[path])
        del(self.open_files[path])
        del(self.flushed[path])
        p = KubePath().parse_path(path)
        self.client.delete_from_cache(p.namespace, p.resource_type, 
            p.object_id, p.action)

    def sync(self, path, dry_run=False):
        if path not in self.open_files:
            return
        self.persist(path, self.open_files[path], dry_run)

    def persist(self, path, data, dry_run=False):
        if path in self.flushed and data == self.flushed[path]:
            return 
        self.flushed[path] = data
        if not dry_run:
            self.client.replace(data)

    def list_files(self, path):
        p = KubePath().parse_path(path)
        if not p.exists(self.client):
            logging.info("path doesn't exist")
            raise FuseOSError(errno.ENOENT)
        if not p.is_dir():
            logging.info("not a directory")
            raise FuseOSError(errno.ENOTDIR)
        if p.object_id is not None: 
            if p.resource_type != 'pod':
                return p.SUPPORTED_ACTIONS
            else:
                return p.SUPPORTED_POD_ACTIONS
        if p.resource_type is not None:
            return self.client.get_entities(p.namespace, p.resource_type)
        if p.namespace is not None:
            return p.SUPPORTED_RESOURCE_TYPES
        return self.client.get_namespaces() # + ['all']

    def getattr(self, path):
        p = KubePath().parse_path(path)
        if path in self.open_files:
            return self._stat_file(self.client, p, len(self.open_files[path]))
        if not p.exists(self.client):
            logging.info("path doesn't exist")
            raise FuseOSError(errno.ENOENT)
        if p.action is not None:
            return self._stat_file(self.client, p)
        return self._stat_dir()

    def read(self, path, size, offset):
        p = KubePath().parse_path(path)
        if not p.is_file():
            raise FuseOSError(errno.ENOENT)
        if path in self.flushed:
            data = self.flushed[path]
        else:
            data = p.do_action(self.client)
        return data[offset:offset + size]




import subprocess
import yaml
import tempfile
import six

from cache import ExpiringCache

class KubernetesClient(object):
    def __init__(self):
        self._cache = ExpiringCache(30)

    def _run_command(self, cmd):
        return subprocess.check_output(' '.join(['kubectl'] + cmd), shell=True)

    def _run_command_and_parse_yaml(self, cmd):
        return yaml.load(self._run_command(cmd))

    def _namespace(self, ns):
        return ['--namespace', ns] if ns != 'all' else ['--all-namespaces']

    def _load_from_cache_or_do(self, key, func):
        cached = self._cache.get(key)
        if cached is not None:
            return cached
        result = func()
        self._cache.set(key, result)
        return result
        
    def get_namespaces(self):
        key = "namespaces"
        cb = self._get_namespaces
        return self._load_from_cache_or_do(key, cb)

    def get_entities(self, ns, entity):
        key = "%s.%s" % (ns, entity)
        cb = lambda: self._get_entities(ns,entity)
        return self._load_from_cache_or_do(key, cb)

    def get_object_in_format(self, ns, entity_type, object, format):
        key = "%s.%s.%s.%s" % (ns, entity_type, object, format)
        cb = lambda: self._get_object_in_format(ns, entity_type, object, format)
        return self._load_from_cache_or_do(key, cb)

    def describe(self, ns, entity_type, object):
        key = "%s.%s.%s.describe" % (ns, entity_type, object)
        cb = lambda: self._describe(ns, entity_type, object)
        return self._load_from_cache_or_do(key, cb)

    def delete_from_cache(self, ns, entity_type, object, format):
        key = "%s.%s.%s.%s" % (ns, entity_type, object, format)
        self._cache.delete(key)

    def logs(self, ns, object):
        key = "%s.pod.%s.logs" % (ns, object)
        cb = lambda: self._logs(ns, object)
        return self._load_from_cache_or_do(key, cb)

    def get_pods(self, ns='default'):
        return self.get_entities(ns, 'pod')
    def get_services(self, ns='default'):
        return self.get_entities(ns, 'svc')
    def get_replication_controllers(self, ns='default'):
        return self.get_entities(ns, 'rc')

    def replace(self, data):
        tmpfile = tempfile.mktemp()
        with open(tmpfile, 'w') as f:
            f.write(data)
        six.print_(self._run_command(('apply -f ' + tmpfile).split()))


    def _get_namespaces(self):
        payload = self._run_command_and_parse_yaml('get ns -o yaml'.split())
        if payload is None or 'items' not in payload:
            return []
        return [ item['metadata']['name'] for item in payload['items']]

    def _get_entities(self, ns, entity):
        payload = self._run_command_and_parse_yaml(['get', entity, '-o', 'yaml'] + self._namespace(ns))
        if payload is None or 'items' not in payload:
            return []
        return [ item['metadata']['name'] for item in payload['items']]

    def _get_object_in_format(self, ns, entity_type, object, format):
        return self._run_command(['get', entity_type, object, '-o', format] + self._namespace(ns))

    def _describe(self, ns, entity_type, object):
        return self._run_command(['describe', entity_type, object] + self._namespace(ns))

    def _logs(self, ns, object):
        return self._run_command(['logs', object] + self._namespace(ns))




from hamcrest import *
import stat
import unittest
from fuse import FuseOSError

from kubefuse.client import KubernetesClient
from kubefuse.path import KubePath
from kubefuse.filesystem import KubeFileSystem

class KubeFileSystemTest(unittest.TestCase):
    def test_getattr_for_namespace(self):
        fs = KubeFileSystem(KubernetesClient())
        path = '/default'
        attr = fs.getattr(path)
        assert_that(attr['st_mode'], is_(stat.S_IFDIR | 0555))
        assert_that(attr['st_nlink'], is_(2))
        assert_that(attr['st_size'], is_(0))
        # NB. time not tested, but whatever

    def test_getattr_for_resource(self):
        fs = KubeFileSystem(KubernetesClient())
        path = '/default/pod'
        attr = fs.getattr(path)
        assert_that(attr['st_mode'], is_(stat.S_IFDIR | 0555))
        assert_that(attr['st_nlink'], is_(2))
        assert_that(attr['st_size'], is_(0))
        # NB. time not tested, but whatever

    def test_getattr_for_object(self):
        client = KubernetesClient()
        pod = client.get_pods()[0]
        fs = KubeFileSystem(client)
        path = '/default/pod/%s' % pod
        attr = fs.getattr(path)
        assert_that(attr['st_mode'], is_(stat.S_IFDIR | 0555))
        assert_that(attr['st_nlink'], is_(2))
        assert_that(attr['st_size'], is_(0))
        # NB. time not tested, but whatever

    def test_getattr_for_action(self):
        client = KubernetesClient()
        pod = client.get_pods()[0]
        fs = KubeFileSystem(client)
        path = '/default/pod/%s/describe' % pod
        attr = fs.getattr(path)
        data = client.describe('default', 'pod', pod)
        assert_that(attr['st_mode'], is_(stat.S_IFREG | 0444))
        assert_that(attr['st_nlink'], is_(1))
        assert_that(attr['st_size'], is_(len(data)))
        # NB. time not tested, but whatever

    def test_getattr_size_for_json_action(self):
        client = KubernetesClient()
        pod = client.get_pods()[0]
        fs = KubeFileSystem(client)
        path = '/default/pod/%s/json' % pod
        attr = fs.getattr(path)
        data = client.get_object_in_format('default', 'pod', pod, 'json')
        assert_that(attr['st_size'], is_(len(data)))

    def test_getattr_size_for_yaml_action(self):
        client = KubernetesClient()
        pod = client.get_pods()[0]
        fs = KubeFileSystem(client)
        path = '/default/pod/%s/yaml' % pod
        attr = fs.getattr(path)
        data = client.get_object_in_format('default', 'pod', pod, 'yaml')
        assert_that(attr['st_size'], is_(len(data)))

    def test_getattr_size_for_describe_action(self):
        client = KubernetesClient()
        pod = client.get_pods()[0]
        fs = KubeFileSystem(client)
        path = '/default/pod/%s/logs' % pod
        attr = fs.getattr(path)
        data = client.logs('default', pod)
        assert_that(attr['st_size'], is_(len(data)))

    def test_getattr_for_nonexistent_path(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        path = '/doesnt-exist'
        assert_that(calling(lambda: fs.getattr(path)), raises(FuseOSError))

    def test_getattr_for_truncated_file(self):
        client = KubernetesClient()
        pod = client.get_pods()[0]
        fs = KubeFileSystem(client)
        path = '/default/pod/%s/json' % pod
        fs.truncate(path, 0)
        attr = fs.getattr(path)
        assert_that(attr['st_size'], is_(0))

    def test_list_files_for_root(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        path = '/'
        files = fs.list_files(path)
        namespaces = client.get_namespaces()
        assert_that(files, contains(*namespaces))
        assert_that(len(files), is_(len(namespaces)))

    def test_list_files_for_namespace(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        path = '/default'
        files = fs.list_files(path)
        assert_that(files, contains_inanyorder('pod', 'svc', 'rc', 'deployments', 'nodes', 'events',
            'limits', 'pv', 'pvc', 'quota', 'endpoints', 'serviceaccounts', 'jobs', 'replicasets',
            'configmaps', 'secrets', 'componentstatuses', 'daemonsets', 'horizontalpodautoscalers', 'ingress'))

    def test_list_files_for_resource(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        path = '/default/pod'
        files = fs.list_files(path)
        pods = client.get_pods()
        assert_that(files, contains(*pods))

    def test_list_files_for_pod(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s' % pod
        files = fs.list_files(path)
        assert_that(files, has_items('describe', 'logs', 'json', 'yaml'))
        assert_that(len(files), is_(4))

    def test_list_files_for_rc(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        rc = client.get_replication_controllers()[0]
        path = '/default/rc/%s' % rc
        files = fs.list_files(path)
        assert_that(files, has_items('describe', 'json', 'yaml'))
        assert_that(len(files), is_(3))

    def test_list_files_for_file_throws_exception(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/describe' % pod
        assert_that(calling(lambda: fs.list_files(path)), raises(FuseOSError))

    def test_list_files_for_nonexistent_path(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        path = '/doesnt-exist'
        assert_that(calling(lambda: fs.list_files(path)), raises(FuseOSError))

    def test_read_describe(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/describe' % pod
        data = fs.read(path, 50000, 0)
        assert_that(data, equal_to(client.describe('default', 'pod', pod)))

    def test_read_logs(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/logs' % pod
        data = fs.read(path, 50000, 0)
        assert_that(data, equal_to(client.logs('default', pod)))

    def test_read_json(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/json' % pod
        data = fs.read(path, 50000, 0)
        assert_that(data, equal_to(client.get_object_in_format('default', 'pod', pod, 'json')))

    def test_read_yaml(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/yaml' % pod
        data = fs.read(path, 50000, 0)
        assert_that(data, equal_to(client.get_object_in_format('default', 'pod', pod, 'yaml')))

    def test_read_length(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/yaml' % pod
        data = fs.read(path, 10, 0)
        ref = client.get_object_in_format('default', 'pod', pod, 'yaml')
        assert_that(data, equal_to(ref[:10]))

    def test_read_offset(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/yaml' % pod
        data = fs.read(path, 10, 5)
        ref = client.get_object_in_format('default', 'pod', pod, 'yaml')
        assert_that(data, equal_to(ref[5:15]))
        assert_that(len(data), is_(10))

    def test_truncate_and_write(self):
        client = KubernetesClient()
        fs = KubeFileSystem(client)
        pod = client.get_pods()[0]
        path = '/default/pod/%s/yaml' % pod
        fs.truncate(path, 0)
        fs.write(path, 'test', 0)
        fs.write(path, 'write', 4)
        fs.sync(path, dry_run=True)
        data = fs.read(path, 1000, 0)
        assert_that(data, is_('testwrite'))




from hamcrest import *
import unittest

from kubefuse.client import KubernetesClient
from kubefuse.path import KubePath

class KubePathTest(unittest.TestCase):
    def test_parse_path_namespace(self):
        kp = KubePath()
        kp.parse_path('/default')
        assert_that(kp.namespace, is_('default'))
        assert_that(kp.resource_type, is_(None))
        assert_that(kp.object_id, is_(None))
        assert_that(kp.action, is_(None))

    def test_parse_path_resource_type(self):
        kp = KubePath()
        kp.parse_path('/default/pod')
        assert_that(kp.namespace, is_('default'))
        assert_that(kp.resource_type, is_('pod'))
        assert_that(kp.object_id, is_(None))
        assert_that(kp.action, is_(None))

    def test_parse_path_object_id(self):
        kp = KubePath()
        kp.parse_path('/default/pod/pod-123')
        assert_that(kp.namespace, is_('default'))
        assert_that(kp.resource_type, is_('pod'))
        assert_that(kp.object_id, is_('pod-123'))
        assert_that(kp.action, is_(None))

    def test_parse_path_action(self):
        kp = KubePath()
        kp.parse_path('/default/pod/pod-123/describe')
        assert_that(kp.namespace, is_('default'))
        assert_that(kp.resource_type, is_('pod'))
        assert_that(kp.object_id, is_('pod-123'))
        assert_that(kp.action, is_('describe'))

    def test_path_exists_pod(self):
        client = KubernetesClient()
        pods = client.get_pods()
        pod1 = pods[0]
        kp = KubePath()
        kp.parse_path('/default/pod/%s/describe' % pod1)
        assert_that(kp.exists(client), is_(True))

    def test_path_exists_svc(self):
        client = KubernetesClient()
        svc = client.get_services()
        svc1 = svc[0]
        kp = KubePath()
        kp.parse_path('/default/svc/%s/describe' % svc1)
        assert_that(kp.exists(client), is_(True))

    def test_path_exists_rc(self):
        client = KubernetesClient()
        rc = client.get_replication_controllers()
        rc1 = rc[0]
        kp = KubePath()
        kp.parse_path('/default/rc/%s/describe' % rc1)
        assert_that(kp.exists(client), is_(True))

    def test_path_exists_invalid_namespace(self):
        client = KubernetesClient()
        kp = KubePath()
        kp.parse_path('/unknown-namespace')
        assert_that(kp.exists(client), is_(False))

    def test_path_exists_invalid_resource(self):
        client = KubernetesClient()
        kp = KubePath()
        kp.parse_path('/default/mylovelyresource')
        assert_that(kp.exists(client), is_(False))

    def test_path_exists_invalid_object_id(self):
        client = KubernetesClient()
        kp = KubePath()
        kp.parse_path('/default/pod/invalid-id')
        assert_that(kp.exists(client), is_(False))

    def test_path_exists_invalid_action(self):
        client = KubernetesClient()
        pods = client.get_pods()
        pod1 = pods[0]
        kp = KubePath()
        kp.parse_path('/default/pod/%s/invalid-action' % pod1)
        assert_that(kp.exists(client), is_(False))



from hamcrest import *
import unittest

from kubefuse.cache import ExpiringCache

class CacheTest(unittest.TestCase):

    def test_cache_get_set(self):
        cache = ExpiringCache(expire_in_seconds = 3600)
        cache.set("key", "value")
        assert_that(cache.get("key"), equal_to("value"))
    
    def test_cache_expired(self):
        cache = ExpiringCache(expire_in_seconds = -10) # expires straight away
        cache.set("key", "value")
        assert_that(cache.get('key'), equal_to(None))

    def test_cache_miss(self):
        cache = ExpiringCache(expire_in_seconds = 10)
        assert_that(cache.get('nonexisting'), equal_to(None))




import logging
import time

class ExpiringCache(object):
    def __init__(self, expire_in_seconds):
        self._cache = {}
        self._timestamps = {}
        self.EXPIRE_IN_SECONDS = expire_in_seconds

    def set(self, key, value):
        t = time.time() + self.EXPIRE_IN_SECONDS
        self._cache[key] = value
        self._timestamps[key] = t
        logging.info("Add key '%s' to cache (expires %d)" % (key, t))

    def get(self, key):
        if key not in self._timestamps:
            return None
        now = time.time()
        expires_at = self._timestamps[key]
        if now < expires_at:
            logging.info("Retrieved '%s' from cache" % key)
            return self._cache[key]
        self.delete(key)
        return None

    def delete(self, key):
        del(self._timestamps[key])
        del(self._cache[key])



#!/usr/bin/env python

import logging
import sys
import os
import errno
import six
from fuse import FUSE, FuseOSError, Operations, LoggingMixIn

from cache import ExpiringCache
from client import KubernetesClient
from path import KubePath
from filesystem import KubeFileSystem

class KubeFuse(LoggingMixIn, Operations):

    def __init__(self, mount):
        self.client = KubernetesClient()
        self.fs = KubeFileSystem(self.client)
        self.fd = 0
        six.print_("Mounted on", mount)

    def readdir(self, path, fh):
        return self.fs.list_files(path)

    def getattr(self, path, fh=None):
        return self.fs.getattr(path)

    def open(self, path, fh):
        self.fd += 1
        self.fs.open(path, fh)
        return self.fd

    def read(self, path, size, offset, fh):
        return self.fs.read(path, size, offset)

    def truncate(self, path, length, fh=None):
        self.fs.truncate(path, length)
        return 0

    def write(self, path, buf, offset, fh):
        written = self.fs.write(path, buf, offset)
        return written

    def flush(self, path, fh):
        self.fs.sync(path)
        return 0

    def release(self, path, fh):
        self.fs.close(path)
        return 0

def main():
    if len(sys.argv) != 2:
        six.print_('usage: %s <mountpoint>' % sys.argv[0])
        exit(1)

    logging.basicConfig(level=logging.INFO)
    fuse = FUSE(KubeFuse(sys.argv[1]), sys.argv[1], foreground=True)

if __name__ == '__main__':
    main()



import json
from datetime import datetime

class KubePath(object):
    def __init__(self, namespace = None, resource_type = None, object_id = None, action = None):
        self.namespace = namespace 
        self.resource_type = resource_type
        self.object_id = object_id
        self.action = None
        self.SUPPORTED_RESOURCE_TYPES = [
            'configmaps',
            'componentstatuses', 
            'daemonsets',
            'deployments',
            'endpoints',
            'events', 
            'horizontalpodautoscalers',
            'ingress',
            'jobs',
            'limits', 
            'nodes', 
            'pod', 
            'pv', 
            'pvc',
            'quota', 
            'rc', 
            'replicasets',
            'secrets',
            'serviceaccounts', 
            'svc', 
        ]
        self.SUPPORTED_ACTIONS = ['describe', 'json', 'yaml']
        self.SUPPORTED_POD_ACTIONS = ['logs'] + self.SUPPORTED_ACTIONS

    def parse_path(self, path):
        if path == '/': return self
        parts = path[1:].split("/")
        self.namespace = parts[0] if len(parts) > 0 else None
        self.resource_type = parts[1] if len(parts) > 1 else None
        self.object_id = parts[2] if len(parts) > 2 else None
        self.action = parts[3] if len(parts) > 3 else None
        return self

    def get_creation_date_for_action_file(self, client):
        if self.action not in ['json', 'yaml', 'describe']:
            return None
        metadata = client.get_object_in_format(
            self.namespace, self.resource_type,
            self.object_id, 'json')
        json_data = json.loads(metadata)
        ts = None
        if 'metadata' in json_data:
            if 'creationTimestamp' in json_data['metadata']:
                timestamp = json_data['metadata']['creationTimestamp']
                if timestamp is not None:
                    ts = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%SZ")
                    ts = int(ts.strftime("%s"))
        return ts

    def is_dir(self):
        return self.action is None

    def is_file(self):
        return not(self.is_dir())

    def exists(self, client):
        if self.namespace is None:
            return True
        namespaces = client.get_namespaces()
        if self.namespace not in namespaces:
            return False
        if self.resource_type is None:
            return True
        if self.resource_type not in self.SUPPORTED_RESOURCE_TYPES:
            return False
        if self.object_id is None:
            return True
        entities = client.get_entities(self.namespace, self.resource_type)
        if self.object_id not in entities:
            return False
        if self.action is None:
            return True
        if self.resource_type == 'pod' and self.action in self.SUPPORTED_POD_ACTIONS:
            return True
        if self.action not in self.SUPPORTED_ACTIONS:
            return False
        return True

    def do_action(self, client):
        ns, rt, oid = self.namespace, self.resource_type, self.object_id
        if self.action == 'describe':
            return client.describe(ns, rt, oid)
        if self.action == 'logs':
            return client.logs(ns, oid)
        if self.action in ['json', 'yaml']:
            return client.get_object_in_format(ns, rt, oid, self.action)

    def get_mode(self):
        return 0o444 if self.action not in ['json', 'yaml'] else 0o666

    def __repr__(self):
        result = ['<']
        if self.action is not None:
            result.append("action %s on" % self.action)
        if self.object_id is not None:
            result.append('object %s' % self.object_id)
        if self.resource_type is not None:
            result.append("of type %s" % self.resource_type)
        if self.namespace is not None:
            result.append('in namespace %s' % self.namespace)
        result.append('>')
        return " ".join(result)



from hamcrest import *
import yaml
import json
import unittest

from kubefuse.client import KubernetesClient

class KubernetesClientTest(unittest.TestCase):
    def test_get_namespaces(self):
        client = KubernetesClient()
        namespaces = client.get_namespaces()
        assert_that(namespaces, has_item('default'))
        assert_that(namespaces, has_item('kube-system'))
        assert_that(len(namespaces), is_(2))

    def test_get_pods(self):
        client = KubernetesClient()
        pods = client.get_pods('default')
        assert_that(len(pods), is_(3))

    def test_get_services(self):
        client = KubernetesClient()
        svc = client.get_services('default')
        assert_that(len(svc), is_(4))

    def test_get_replication_controllers(self):
        client = KubernetesClient()
        rc = client.get_replication_controllers('default')
        assert_that(len(rc), is_(3))

    def test_get_object_in_yaml_format(self):
        client = KubernetesClient()
        pods = client.get_pods("default")
        pod = client.get_object_in_format('default', 'pod', pods[0], 'yaml')
        result = yaml.load(pod)
        assert_that(result['metadata']['name'], is_(pods[0]))

    def test_get_object_in_json_format(self):
        client = KubernetesClient()
        pods = client.get_pods("default")
        pod = client.get_object_in_format('default', 'pod', pods[0], 'json')
        result = json.loads(pod)
        assert_that(result['metadata']['name'], is_(pods[0]))

    def test_describe(self):
        client = KubernetesClient()
        pods = client.get_pods("default")
        describe = client.describe('default', 'pod', pods[0])
        assert_that(describe, contains_string(pods[0]))

    def test_logs(self):
        client = KubernetesClient()
        pods = client.get_pods("default")
        describe = client.logs('default', pods[0])
        assert_that(describe, contains_string(pods[0]))



from setuptools import setup, find_packages

setup(
    name = "kubefuse",
    version = "0.6.0",
    packages = find_packages(),
    author = "Bart Spaans",
    author_email = "bart.spaans@gmail.com",
    url = "https://github.com/bspaans/kubefuse",
    description = "A FUSE file system for Kubernetes",
    classifiers = [
        'Programming Language :: Python',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: System',
        'Topic :: System :: Clustering',
        'Topic :: System :: Filesystems',
    ],
    license="Apache License v2",
    entry_points = {
        'console_scripts': [
            'kubefuse = kubefuse.kubefuse:main'
        ]
    }
 )
